/*
*********************************************
*											*
*		UTC TAI sample program				*
*											*
*********************************************
*/
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <math.h>
#include <time.h>
#include "atime.h"

#define		VERSION			"atime Ver 2016/10/12\nUTC -> TT Conversion"
#define		REAPSEC_FILE	"dAT.tbl"
REAPSEC *pdAT;


/*
=============================================
=		display help						=
=============================================
*/
void disp_help(void)
{
	puts(VERSION);
	puts("atime --- PC(UTC) system time");
	puts("atime Year [Month Day Hour Minute Second] --- Any UTC time");
}

/*
=============================================
=		Read leapsec. table for TT			=
=============================================
*/
int read_leepsec_table(char *pFileName)
{
	FILE *fp;
	REAPSEC *ptbl;

	int ndTCount;
	double sec;
	char szDmy[80];

	/*
	-------------------------------------------
			check parameter line
	-------------------------------------------
	*/
	/* Open File */
	fp = fopen(pFileName, "r");
	if(fp == NULL){
		printf("%s cannot open\n", pFileName);
		return -1;
	}
	/* calc. size */
	ndTCount = 0;
	while(!feof(fp)){
		if (fgets(szDmy, sizeof(szDmy), fp) == NULL){
			break;
		}
		ndTCount++;
	}

	/* memory allocate */
	pdAT = (REAPSEC *)malloc(ndTCount * sizeof(REAPSEC));
	if (pdAT == NULL){
		fclose(fp);
		puts("Memory allocate error!");
		return -1;
	}

	/*
	-------------------------------------------
			load reap second data
	-------------------------------------------
	*/
	/* Read data */
	ptbl = pdAT;
	fseek(fp, 0, SEEK_SET);
	while(!feof(fp)){
		if (fgets(szDmy, sizeof(szDmy), fp) == NULL){
			break;
		}
		ptbl->dJD = atof(szDmy);
		sec = atof(&szDmy[10]);
		ptbl->dTime = (sec + DTAI_TT) / DAYSEC;
		ptbl++;
	}
	fclose(fp);

	return 0;
}

int main(int argc, char *argv[])
{
	time_t timer;
	struct tm *t_st;

	DATETIME dtime = {0, 1, 1, 0, 0, 0};
	double dJD_TT;
	double dDAT;
	double dDay;
	char *pHead;
	char cTemp;
	int nTTMode;
	int i;

	/* headder */
	char szDT[] = "dT(ET-UT)";
	char szET[] = "ET";
	char szUT[] = "UT";

	char szDTT[] = "dTT(TT-UTC)";
	char szTT[] = "TT";
	char szUTC[] = "UTC";

	/* Set arg data */
	if (argc == 1){
		// get now time
		time(&timer);
		t_st = gmtime(&timer);
		dtime.nYear = t_st->tm_year+1900;
		dtime.nMonth = t_st->tm_mon+1;
		dtime.nDay = t_st->tm_mday;
		dtime.nHour = t_st->tm_hour;
		dtime.nMin = t_st->tm_min;
		dtime.dSec = t_st->tm_sec;
	}else{
		i = argc - 1;
		if (i != 0){
			/* check help(-h or -?) */
			if (argv[1][0] == '-'){
				cTemp = argv[1][1];
				if ((cTemp == 'h') || (cTemp == 'H') || (cTemp == '?')){
					disp_help();
					return -1;
				}
			}
			dtime.nYear = atoi(argv[1]);
			i--;
		}
		if (i != 0){
			dtime.nMonth = atoi(argv[2]);
			i--;
		}
		if (i != 0){
			dtime.nDay = atoi(argv[3]);
			i--;
		}
		if (i != 0){
			dtime.nHour = atoi(argv[4]);
			i--;
		}
		if (i != 0){
			dtime.nMin = atoi(argv[5]);
			i--;
		}
		if (i != 0){
			dtime.dSec = atof(argv[6]);
			i--;
		}
	}

	/* DAT table read */
	pdAT = NULL;
	if (read_leepsec_table(REAPSEC_FILE) != 0){
		return -1;
	}
	set_dtt_table(pdAT);

	/* UTC to TT or ET */
	nTTMode = 0;
	if (dtime.nYear >= 1972){
		nTTMode = -1;
	}
	dDAT = utc_tt(&dtime, &dJD_TT);
	if (nTTMode != 0){
		pHead = szDTT;
	}else{
		pHead = szDT;
	}
	printf("%s %f\n", pHead, dDAT);

	/* display TT date & time */
	juian_date(dJD_TT, &dtime.nYear, &dtime.nMonth, &dDay);
	dtime.nDay = (int)floor(dDay);
	dDay -= dtime.nDay;
	dayfrac_time(dDay, &dtime.nHour, &dtime.nMin, &dtime.dSec);
	if (nTTMode != 0){
		pHead = szTT;
	}else{
		pHead = szET;
	}
	printf("%s %04d/%02d/%02d %02d:%02d:%07.4f\n", pHead, dtime.nYear, dtime.nMonth, dtime.nDay, dtime.nHour, dtime.nMin, dtime.dSec);

	/* TT to UTC */
	tt_utc(dJD_TT, &dtime);
	if (nTTMode != 0){
		pHead = szUTC;
	}else{
		pHead = szUT;
	}
	printf("%s %04d/%02d/%02d %02d:%02d:%07.4f\n", pHead, dtime.nYear, dtime.nMonth, dtime.nDay, dtime.nHour, dtime.nMin, dtime.dSec);

	/* Memory free */
	free(pdAT);
	return 0;
}
